-- Create the addon
Fizzle = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceDB-2.0", "AceConsole-2.0", "AceModuleCore-2.0")
Fizzle:SetModuleMixins("AceEvent-2.0", "AceHook-2.1")
Fizzle:RegisterDB("FizzleDB")
Fizzle:RegisterDefaults('profile', {
	Percent = true,
	Border = true,
	Invert = false,
	HideText = false,
	DisplayWhenFull = true,
})

local fontSize = 12
local abacus = AceLibrary("Abacus-2.0")
local crayon = AceLibrary("Crayon-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Fizzle")
local _G = getfenv(0)
local sformat = string.format

-- Flag to check if the borders were created or not
local bordersCreated = false

function Fizzle:OnInitialize()
	-- Register Chat Commands
	self.options = {
		desc = L["Displays Item Quality and Durability in the character frame."],
		type = "group",
		args = {
			percent = {
				name = L["Percent"],
				desc = L["Toggle percentage display."],
				type = "toggle",
				get = function() return self.db.profile.Percent end,
				set = function() self.db.profile.Percent = not self.db.profile.Percent self:UpdateItems() end,
			},
			border = {
				name = L["Border"],
				desc = L["Toggle quality borders."],
				type = "toggle",
				get = function() return self.db.profile.Border end,
				set = function() self.db.profile.Border = not self.db.profile.Border self:BorderToggle() end,
			},
			invert = {
				name = L["Invert"],
				desc = L["Show numbers the other way around. Eg. 0% = full durability , 100 = no durability."],
				type = "toggle",
				get = function() return self.db.profile.Invert end,
				set = function() self.db.profile.Invert = not self.db.profile.Invert self:UpdateItems() end,
			},
			hidetext = {
				name = L["Hide Text"],
				desc = L["Hide durability text."],
				type = "toggle",
				get = function() return self.db.profile.HideText end,
				set = function() self.db.profile.HideText = not self.db.profile.HideText self:UpdateItems() end,
			},
			showfull = {
				name = L["Show Full"],
				desc = L["Show durability when full."],
				type = "toggle",
				get = function() return self.db.profile.DisplayWhenFull end,
				set = function() self.db.profile.DisplayWhenFull = not self.db.profile.DisplayWhenFull self:UpdateItems() end,
			}
		}
	}

	Fizzle:RegisterChatCommand({ "/fizzle", "/fizz" }, self.options)
end

function Fizzle:OnEnable()
	self:SecureHook("CharacterFrame_OnShow")
	self:SecureHook("CharacterFrame_OnHide")
	if not bordersCreated then
		self:MakeTypeTable()
	end
end

function Fizzle:OnDisable()
	for _, item in ipairs(self.items) do
		_G[item.slot .. "FizzleS"]:SetText("")
	end
	self:HideBorders()
end

function Fizzle:CreateBorder(slottype, slot, name, hasText)
	local gslot = _G[slottype..slot.."Slot"]
	if gslot then
		-- Create border
		local border = gslot:CreateTexture(slot .. name .. "B", "OVERLAY")
		border:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
		border:SetBlendMode("ADD")
		border:SetAlpha(0.75)
		border:SetHeight(68)
		border:SetWidth(68)
		border:SetPoint("CENTER", gslot, "CENTER", 0, 1)
		border:Hide()

		-- Check if we need a text field creating
		if hasText then
			local str = gslot:CreateFontString(slot .. name .. "S", "OVERLAY")
			local font, _, flags = NumberFontNormal:GetFont()
			str:SetFont(font, fontSize, flags)
			str:SetPoint("CENTER", gslot, "BOTTOM", 0, 8)
		end
	end
end

function Fizzle:MakeTypeTable()
	-- Table of item types and slots.  Thanks Tekkub.
	self.items = self.items or {
		"Head",
		"Shoulder",
		"Chest",
		"Waist",
		"Legs",
		"Feet",
		"Wrist",
		"Hands",
		"MainHand",
		"SecondaryHand",
		"Ranged",
	}
         
	-- Items without durability but with some quality, needed for border colouring.
	self.nditems = self.nditems or {
		"Ammo",
		"Neck",
		"Back",
		"Finger0",
		"Finger1",
		"Trinket0",
		"Trinket1",
		"Relic",
		"Tabard",
	}
         
	for _, item in ipairs(self.items) do
		self:CreateBorder("Character", item, "Fizzle", true)
	end

	-- Same again, but for ND items, and only creating a border
	for _, nditem in ipairs(self.nditems) do
		self:CreateBorder("Character", nditem, "Fizzle", false)
	end
end

function Fizzle:UpdateItems()
	-- Don't update unless the charframe is open.
	-- No point updating what we can't see.
	if CharacterFrame:IsVisible() then
		-- Go and set the durability string for each slot that has an item equipped that has durability.
		-- Thanks Tekkub again for the base of this code.
		for _, item in ipairs(self.items) do
			local id, _ = GetInventorySlotInfo(item .. "Slot")
			local str = _G[item.."FizzleS"]
			local v1, v2 = GetInventoryItemDurability(id)
			v1, v2 = tonumber(v1) or 0, tonumber(v2) or 0
			local percent = v1 / v2 * 100

			if (((v2 ~= 0) and ((percent ~= 100) or self.db.profile.DisplayWhenFull)) and not self.db.profile.HideText) then
				local text
			
				-- Colour our string depending on current durability percentage
				str:SetTextColor(crayon:GetThresholdColor(v1/v2))

				if self.db.profile.Invert then
					v1 = v2 - v1
					percent = 100 - percent
				end

				-- Are we showing the % or raw cur/max
				if self.db.profile.Percent then
					text = sformat("%d%%", percent)
				else
					text = v1.."/"..v2
				end

				str:SetText(text)
			else
				-- No durability in slot, so hide the text.
				str:SetText("")
			end
             
			--Finally, colour the borders
			if self.db.profile.Border then
				self:ColourBorders(id, item)
			end
		end
         
		-- Colour the borders of ND items
		if self.db.profile.Border then
			self:ColourBordersND()
		end
	end
end

function Fizzle:CharacterFrame_OnShow()
	if not self:IsEventRegistered("UNIT_INVENTORY_CHANGED") then
		self:RegisterEvent("UNIT_INVENTORY_CHANGED", "UpdateItems")
	end
	self:UpdateItems()
end

function Fizzle:CharacterFrame_OnHide()
	if self:IsEventRegistered("UNIT_INVENTORY_CHANGED") then
		self:UnregisterEvent("UNIT_INVENTORY_CHANGED", "UpdateItems")
	end
end

-- Border colouring split into two functions so I only need to iterate over each table once.
-- Border colouring for items with durability.
function Fizzle:ColourBorders(slotID, rawslot)
	local quality = GetInventoryItemQuality("player", slotID)
	if quality then
		local r, g, b, _ = GetItemQualityColor(quality)
		_G[rawslot.."FizzleB"]:SetVertexColor(r, g, b)
		_G[rawslot.."FizzleB"]:Show()
	else
		_G[rawslot.."FizzleB"]:Hide()
	end
end

-- Border colouring for items without durability
function Fizzle:ColourBordersND()
	for _, nditem in ipairs(self.nditems) do
		if _G["Character"..nditem.."Slot"] then
			local slotID, _ = GetInventorySlotInfo(nditem .. "Slot")
			local quality = GetInventoryItemQuality("player", slotID)
			if quality then
				local r, g, b, _ = GetItemQualityColor(quality)
				_G[nditem.."FizzleB"]:SetVertexColor(r, g, b)
				_G[nditem.."FizzleB"]:Show()
			else
				_G[nditem.."FizzleB"]:Hide()
			end
		end
	end
end

-- Toggle the border colouring
function Fizzle:BorderToggle()
	if not self.db.profile.Border then
		self:HideBorders()
	else
		self:UpdateItems()
	end
end

-- Hide quality borders
function Fizzle:HideBorders()
	for _, item in ipairs(self.items) do
		local border = _G[item.."FizzleB"]
		if border then
			border:Hide()
		end
	end
	
	for _, nditem in ipairs(self.nditems) do
		local border = _G[nditem.."FizzleB"]
		if border then
			border:Hide()
		end
	end
end
